@@ -14,9 +14,9 @@
 #'statistics. Because of this \code{matrixpls.plspm} is substantially less efficient than the \code{\link{matrixpls}}
 #'function.
 #'
-#'The argument \code{inner_matrix} is a matrix of zeros and ones that indicates
+#'The argument \code{path_matrix} is a matrix of zeros and ones that indicates
 #'the structural relationships between composites. This must be a lower
-#'triangular matrix. \code{inner_matrix} will contain a 1 when column \code{j}
+#'triangular matrix. \code{path_matrix} will contain a 1 when column \code{j}
 #'affects row \code{i}, 0 otherwise. \cr
 #'
 #'@inheritParams plspm::plspm
@@ -32,21 +32,21 @@
 #'@example example/matrixpls.plspm-example.R
 
 matrixpls.plspm <-
-function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid", 
-         scaled = TRUE, tol = 0.00001, iter = 100, boot.val = FALSE, 
-         br = NULL, plsr = FALSE, dataset = TRUE){
+function(Data, path_matrix, blocks, modes = NULL, scheme = "centroid", 
+         scaled = TRUE, tol = 0.000001, maxiter = 100, boot.val = FALSE, 
+         br = NULL, dataset = TRUE){
     
 		library(plspm)
 		library(psych)
 		library(boot)
 		library(parallel)
-		
+				
 		# =======================================================
 		# checking arguments
 		# =======================================================
-		params = get_params(x=Data, inner=inner_matrix, outer=outer_list, modes=modes, 
-										 scheme=scheme, scaled=scaled, tol=tol, iter=iter,
-										 boot.val=boot.val, br=br, plsr=plsr, dataset=dataset)
+		params = get_params(x=Data, inner=path_matrix, outer=blocks, modes=modes, 
+										 scheme=scheme, scaled=scaled, tol=tol, maxiter=maxiter,
+										 boot.val=boot.val, br=br, plsr=FALSE, dataset=dataset)
 
 		
 		#
@@ -145,7 +145,7 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 				tryCatch(
 					matrixpls(S_boot, model = nativeModel, W.mod = W.mod, parameterEstimator = parameterEstimator,
 										outerEstimators = outerEstimators, innerEstimator = innerEstimator,
-										tol = params$tol, iter = params$iter, convCheck = convCheck,
+										tol = params$tol, iter = params$maxiter, convCheck = convCheck,
 										validateInput = FALSE), 
 					
 					error = function(e){
@@ -161,7 +161,7 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 		else{
 			matrixpls.res <- matrixpls(S, model = nativeModel, W.mod = W.mod, parameterEstimator = parameterEstimator,
 																 outerEstimators = outerEstimators, innerEstimator = innerEstimator,
-																 tol = params$tol, iter = params$iter, convCheck = convCheck,
+																 tol = params$tol, iter = params$maxiter, convCheck = convCheck,
 																 validateInput = FALSE)
 		}
 		
@@ -188,10 +188,11 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 		# Composite values, fittes values, and intercepts
 		
 		lvScores <- dataToUse %*% t(W) * sdv
-		lvScoreFittedValues <- lvScores %*% t(beta)
-		intercepts <- apply(lvScores-lvScoreFittedValues,2,mean)
 		rownames(lvScores) <- 1:nrow(lvScores)
 		
+		lvScores_std <- apply(lvScores, 2, scale) * sdv
+		rownames(lvScores_std) <- 1:nrow(lvScores_std)
+		
 		
 		Modes <- ifelse(params$modes == "A", "Reflective","Formative")
 		
@@ -210,20 +211,53 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 			return(ret)
 		}, simplify= FALSE)
 		
+		outer.mod.dataframe <- do.call(rbind,sapply(lvNames,function(lvName){
+			temp <- outer.mod[[lvName]]
+			data.frame(name = rownames(temp),
+								 block = lvName,
+								 weight = temp[,1],
+								 loading = temp[,2],
+								 communality = temp[,3],
+								 redundancy = temp[,4])
+		}, simplify= FALSE))
+		
+		rownames(outer.mod.dataframe) <- NULL
+		
+		lvScoreFittedValues <- lvScores_std %*% t(beta)
+		intercepts <- apply(lvScores_std-lvScoreFittedValues,2,mean)
+		
 		inner.mod <- sapply(lvNames[!exogenousLVs], function(lvName){
+			
+			# Recalculate the regressions to get standard errors and intercepts
 			row <- which(lvNames == lvName)
 			regressors <- beta[row,]!=0
-			ret <- data.frame(concept =c ("R2","Intercept",paste("path_",lvNames[regressors],sep="")),
-												 value = c(R2[row],intercepts[row],beta[row,regressors]))
 			
-			rownames(ret) <- NULL
+			beta <- c(intercepts[lvName],beta[row,regressors]) 
+
+			# Calculating of SEs adapted from mat.regress from the psych package
+			
+			if(sum(regressors) == 1){
+				uniq <- 1
+			}
+			else{
+				uniq <- (1-smc(C[regressors,regressors]))
+			}
+			df <- nrow(params$x)-sum(regressors)-1
+			se <- sqrt((1-R2[row])/(df))*c(1,sqrt(1/uniq))
+			tvalue <- beta/se
+			prob <- 2*(1- pt(abs(tvalue),df))
+			
+			ret <- cbind(beta, se, tvalue, prob)
+
+			rownames(ret) <- c("Intercept",lvNames[regressors])
+			colnames(ret) <- c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
 			
 			return(ret)
 		}, simplify= FALSE)
 		
 		pathIndices <- lower.tri(nativeModel$inner)
 		
-		pathLabels <- paste(colnames(nativeModel$inner)[col(nativeModel$inner)[pathIndices]],"->",
+		pathLabels <- paste(colnames(nativeModel$inner)[col(nativeModel$inner)[pathIndices]]," -> ",
 												rownames(nativeModel$inner)[row(nativeModel$inner)[pathIndices]], sep="")
 		
 		if(params$boot.val){
@@ -297,24 +331,38 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 		blocks <- unlist(lapply(params$outer, length))
 		names(blocks) <- lvNames
 		
-		model <- list(IDM=params$inner,
-									blocks=blocks,
-									scheme=switch(scheme, "centroid"="centroid", "factor"="factor", "path"="path"),
-									modes=modes, scaled=params$scaled, 
-									boot.val=params$boot.val, plsr=params$plsr, obs=nrow(params$x), br=params$br, 
-									tol=params$tol, iter=params$iter,
-									n.iter=attr(matrixpls.res,"iterations"), outer=params$outer)
+		blocksList <- params$outer
+		names(blocksList) <- lvNames
 		
+		mvsToUse <- unlist(params$outer)
+			
+		model <- list(IDM=params$inner,
+									blocks= blocksList,
+									specs = list(scaling = NULL,
+															 modes = modes, 
+															 scheme=switch(scheme, "centroid"="centroid", "factor"="factor", "path"="path"),
+															 scaled=params$scaled, 
+															 tol=params$tol, 
+															 maxiter = params$maxiter,
+															 plscomp = NULL),
+									iter = attr(matrixpls.res,"iterations") + 1,
+ 									boot.val=params$boot.val, 
+									br=params$br, 
+									gens = list(obs=nrow(params$x),
+															obs_names=rownames(params$x),
+															mvs=length(mvsToUse),
+															mvs_names = colnames(params$x)[mvsToUse],
+															lvs = length(lvNames),
+															lvs_names = lvNames))
+		
+	
 		if(params$dataset){
 			data <- as.matrix(params$x[,unlist(params$outer)])
 		}
 		else{
 			data <- FALSE
 		}
-		
-		lvScores_std <- apply(lvScores, 2, scale) * sdv
-		rownames(lvScores_std) <- 1:nrow(lvScores_std)
-				
+						
 		Windices <-W!=0
 		out.weights <- W[Windices] * sdv
 		names(out.weights) <- colnames(W)[col(W)[Windices]]
@@ -330,12 +378,10 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 		}, simplify= FALSE)
 		
 		
-		inner.sum <- data.frame(LV.Type = ifelse(exogenousLVs,"Exogen","Endogen"),
-														Measure = abbreviate(Modes, 5),
-														MVs = blocks,
-														R.square = R2, 
-														Av.Commu = sapply(outer.mod,function(x){mean(x[,"communal"])}), 
-														Av.Redun = sapply(outer.mod,function(x){mean(x[,"redundan"])}), 
+		inner.sum <- data.frame(Type = ifelse(exogenousLVs,"Exogenous","Endogenous"),
+														R2 = R2, 
+														Block_Communality = sapply(outer.mod,function(x){mean(x[,"communal"])}), 
+														Mean_Redundancy = sapply(outer.mod,function(x){mean(x[,"redundan"])}), 
 														AVE = ifelse(params$modes == "A",
 																				 sapply(outer.mod,function(x){mean(x[,"std.loads"]^2)}),
 																				 0))
@@ -343,11 +389,11 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 		effs <- effects(matrixpls.res)
 				
 		effects <- data.frame(relationships = pathLabels,
-														dir.effects = effs$Direct[pathIndices[-1,]],
-													ind.effects = effs$Indirect[pathIndices[-1,]], 
-													tot.effects = effs$Total[pathIndices[-1,]])
+														direct = effs$Direct[pathIndices[-1,]],
+													indirect = effs$Indirect[pathIndices[-1,]], 
+													total = effs$Total[pathIndices[-1,]])
 
-		unidim <- data.frame(Type.measure = Modes,
+		unidim <- data.frame(Mode = params$modes,
 												 MVs = blocks,
 												 C.alpha = ifelse(params$modes == "A",
 												 								 sapply(params$outer,function(indices){
@@ -376,23 +422,31 @@ function(Data, inner_matrix, outer_list, modes = NULL, scheme = "centroid",
 		gof <- GoF(matrixpls.res)
 		class(gof) <- "numeric"
 		
-		res = list(outer.mod = outer.mod, 
-							 inner.mod = inner.mod, 
-							 latents = lvScores_std, 
-							 scores = lvScores,
-							 out.weights = out.weights, 
-							 loadings = loadings, 
-							 path.coefs = beta, 
-							 r.sqr = R2,
-							 outer.cor = outer.cor, 
-							 inner.sum = inner.sum, 
+		# Crossloadings are the IC matrix
+
+		crossloadings <- cbind(outer.mod.dataframe[,1:2],t(IC)/sqrt(diag(S)))
+		rownames(crossloadings) <- NULL
+		
+		res = list(outer_model = outer.mod.dataframe, 
+							 inner_model = inner.mod, 
+							 path_coefs = beta, 
+							 scores = lvScores_std,
+							 crossloadings = crossloadings,
+							 inner_summary = inner.sum, 
 							 effects = effects,
 							 unidim = unidim, 
 							 gof = gof, 
 							 boot = boot, 
-							 data = data, 
+							 data = as.data.frame(data), 
+							 manifests = scale(data, scale=FALSE),
 							 model = model)
 		
+#		out.weights = out.weights, 
+#		loadings = loadings, 
+#		r.sqr = R2,
+#		outer.cor = outer.cor, 
+		#							 latents = lvScores_std, 
+		
 		class(res) = "plspm"
 
 		return(res)
@@ -522,7 +576,7 @@ get_plsr1 <-function(C, nc=NULL, scaled=TRUE)
 
 get_params <-
 	function(x, inner, outer, modes=NULL, scheme="centroid", scaled=TRUE,
-					 boot.val=FALSE, br=NULL, plsr=FALSE, tol=0.00001, iter=100, dataset=TRUE)
+					 boot.val=FALSE, br=NULL, plsr=FALSE, tol=0.00001, maxiter=100, dataset=TRUE)
 	{
 		# =======================================================
 		# checking arguments
@@ -596,9 +650,9 @@ get_params <-
 			warning("Warning: Invalid argument 'tol'. Default 'tol=0.00001' is used.")   
 			tol <- 0.00001
 		} 
-		if (mode(iter)!="numeric" || length(iter)!=1 || iter<100) {
-			warning("Warning: Invalid argument 'iter'. Default 'iter=100' is used.")   
-			iter <- 100
+		if (mode(maxiter)!="numeric" || length(maxiter)!=1 || maxiter<100) {
+			warning("Warning: Invalid argument 'maxiter'. Default 'maxiter=100' is used.")   
+			maxiter <- 100
 		} 
 		if (!is.logical(dataset)) 
 			dataset <- TRUE
@@ -618,7 +672,7 @@ get_params <-
 			br = br,
 			plsr = plsr,
 			tol = tol,
-			iter = iter,
+			maxiter = maxiter,
 			dataset = dataset)
 		res
 }
